/* 
   phpass.c 
   
   Edited by Milen Rangelov <gat3way@gat3way.eu> to optimize for hash cracking.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */




#define _GNU_SOURCE
#define _XOPEN_SOURCE 600

#include <stdio.h>
#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <stdint.h>
#include "plugin.h"


extern char *pacrypt_private(char *password, char *setting, char *out);



static char *itoa64 =
	"./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

static void encode64(char *dst, char *src, int count)
{
	int i, value;

	i = 0;
	do {
		value = (unsigned char)src[i++];
		*dst++ = itoa64[value & 0x3f];
		if (i < count)
			value |= (unsigned char)src[i] << 8;
		*dst++ = itoa64[(value >> 6) & 0x3f];
		if (i++ >= count)
			break;
		if (i < count)
			value |= (unsigned char)src[i] << 16;
		*dst++ = itoa64[(value >> 12) & 0x3f];
		if (i++ >= count)
			break;
		*dst++ = itoa64[(value >> 18) & 0x3f];
	} while (i < count);
}

char *pacrypt_private(char *password, char *setting, char *out)
{
	char output[35];
	char hash[17];
	char *p, *salt;
	int count_log2, length, count;
	char *bigbuf = alloca(200);
	int bbl;

	strcpy(output, "*0");
	if (!strncmp(setting, output, 2))
		output[1] = '1';

	if (strncmp(setting, "$P$", 3))
	    if (strncmp(setting, "$H$", 3))

		return "";

	p = strchr(itoa64, setting[3]);
	if (!p)
		return "";
	count_log2 = p - itoa64;
	if (count_log2 < 7 || count_log2 > 30)
		return "";

	salt = setting + 4;
	if (strlen(salt) < 8)
		return "";

	length = strlen(password);


	
	bbl = 0;
	memcpy(bigbuf, salt, 8);
	bbl+=8;
	memcpy(bigbuf+bbl, password, length);
	bbl+=length;
	hash_md5(bigbuf, hash, bbl);
	bbl = 0;
	count = 1 << count_log2;
	do {
		memcpy(bigbuf, hash, 16);
		bbl+=16;
		memcpy(bigbuf+bbl, password, length);
		bbl+=length;
		hash_md5(bigbuf, hash, bbl);
		bbl = 0;
	} while (--count);

	memcpy(out, setting, 12);
	encode64(&out[12], hash, 16);

	return out;
}

